// Generated by CoffeeScript 1.12.7
(function() {
  var OperationType, XMLUserDataHandler;

  OperationType = require('./OperationType');

  module.exports = XMLUserDataHandler = (function() {
    function XMLUserDataHandler() {}

    XMLUserDataHandler.prototype.handle = function(operation, key, data, src, dst) {};

    return XMLUserDataHandler;

  })();

}).call(this);
